@@ -5,9 +5,13 @@ import time
 from abc import ABCMeta
 
 from django.utils.translation import ugettext_lazy as _
+from django.db.models import fields
 from django.db.models.fields import NOT_PROVIDED
 from django.core import exceptions, validators
 from django.utils.encoding import force_unicode
+from django.utils import timezone, datetime_safe
+from django.forms import fields as formfields
+from django.conf import settings
 
 from neo4jrestclient.client import NotFoundError
 
@@ -18,13 +22,15 @@ from .. import connections
 from neo4django.validators import validate_array, validate_str_array,\
         validate_int_array, ElementValidator
 from neo4django.utils import AttrRouter, write_through
+from neo4django.decorators import borrows_methods
 from neo4django.constants import ERROR_ATTR
 
-from dateutil.tz import tzutc, tzoffset
-
 MIN_INT = -9223372036854775808
 MAX_INT = 9223372036854775807
 
+FIELD_PASSTHROUGH_METHODS = ('formfield',)
+
+@borrows_methods(fields.Field, FIELD_PASSTHROUGH_METHODS)
 class Property(object):
     """Extend to create properties of specific types."""
     # This class borrows heavily from Django 1.3's django.db.models.field.Field
@@ -38,13 +44,13 @@ class Property(object):
         'blank': _(u'This property cannot be blank.'),
     }
 
-    def __init__(self, indexed=False, indexed_fulltext=False,
-                 indexed_range=False, indexed_by_member=False,
-                 has_own_index=False, unique=False, name=None, editable=True,
-                 null=True, blank=True, validators=[], choices=None,
-                 error_messages=None, required=False, serialize=True,
-                 auto=False, metadata={}, auto_default=NOT_PROVIDED,
-                 default=NOT_PROVIDED, **kwargs):
+    def __init__(self, verbose_name=None, name=None, help_text=None,
+                 indexed=False, indexed_fulltext=False, indexed_range=False,
+                 indexed_by_member=False, has_own_index=False, unique=False,
+                 editable=True, null=True, blank=True, validators=[],
+                 choices=None, error_messages=None, required=False,
+                 serialize=True, auto=False, metadata={},
+                 auto_default=NOT_PROVIDED, default=NOT_PROVIDED, **kwargs):
         if unique and not indexed:
             raise ValueError('A unique property must be indexed.')
         if auto and auto_default == NOT_PROVIDED:
@@ -56,6 +62,10 @@ class Property(object):
         self.indexed_by_member = indexed_by_member
         self.has_own_index = has_own_index
         self.unique = unique
+        # we don't support this uniqueness granularity
+        self.unique_for_date = False
+        self.unique_for_month = False
+        self.unique_for_year = False
         self.editable = editable
         self.blank = blank
         self.null = null
@@ -65,7 +75,10 @@ class Property(object):
         self.meta = metadata
         self._default = default
 
-        self.__name = name
+        self.name = self.__name = name
+        self.attname = self.name
+        self.verbose_name = verbose_name
+        self.help_text = help_text
 
         self.choices = choices or []
 
@@ -77,10 +90,6 @@ class Property(object):
         messages.update(error_messages or {})
         self.error_messages = messages
 
-    @property
-    def attname(self):
-        return self.__name
-
     @property
     def default(self):
         self.get_default()
@@ -205,6 +214,7 @@ class Property(object):
     def pre_save(self, model_instance, add, attname):
         pass
 
+@borrows_methods(fields.Field, ('save_form_data',))
 class BoundProperty(AttrRouter):
     rel = None
     primary_key = False
@@ -243,11 +253,24 @@ class BoundProperty(AttrRouter):
                      'meta',
                      'MAX',
                      'MIN',
+                     #form-related properties
+                     'editable',
+                     'blank',
+                     'formfield',
+                     'unique_for_date',
+                     'unique_for_month',
+                     'unique_for_year',
                     ], self._property)
 
         self.__class = cls
         self.__propname = propname
         self.__attname = attname
+
+        # TODO - i don't know why, but this is the final straw. properties
+        # and boundproperties need to be merged, the coupling is ridiculous
+        self._property.name = propname
+        self._property.attname = attname
+
         properties = self._properties_for(cls)
         properties[self.name] = self # XXX: weakref
 
@@ -315,13 +338,14 @@ class BoundProperty(AttrRouter):
             prop = properties.get(k, None)
             if prop:
                 #XXX duplicates __get_value()...
-                values[k] = prop.from_neo(v)
+                values[k] = prop.to_python(v)
         #XXX this relies on neo4jrestclient private implementation details
         if clear:
             instance.node._dic['data'].clear()
         instance.node._dic['data'].update(new_val_dict)
 
-    NodeModel._update_values_from_dict = staticmethod(_update_values_from_dict) #TODO this needs to be revised
+    #TODO this needs to be revised
+    NodeModel._update_values_from_dict = staticmethod(_update_values_from_dict) 
     del _update_values_from_dict
 
     def _save_(instance, node, node_is_new):
@@ -347,19 +371,20 @@ class BoundProperty(AttrRouter):
                 value = values[key]
                 prop.clean(value, instance)
                 value = prop.pre_save(node, node_is_new, prop.name) or value
-                if not value in validators.EMPTY_VALUES or getattr(prop._property, "use_string", False):
+                if (not value in validators.EMPTY_VALUES or
+                    getattr(prop._property, "use_string", False)):
                     #should already have errored if self.null==False
                     value = prop.to_neo(value)
-                values[key] = value
                 prop_dict['value'] = value
                 if prop.indexed:
                     indexed_values = prop_dict['values_to_index'] = []
                     prop_dict['unique'] = bool(prop.unique)
                     if value is not None:
-                        indexed_values.append(prop.to_neo_index(value))
+                        indexed_values.append(prop.to_neo_index(values[key]))
                         if prop.indexed_by_member:
                             for m in value:
                                 indexed_values.append(prop.member_to_neo_index(m))
+                values[key] = value
         script = '''
         node=g.v(nodeId);
         results = Neo4Django.updateNodeProperties(node, propMap);
@@ -368,16 +393,21 @@ class BoundProperty(AttrRouter):
         script_rv= conn.gremlin_tx(script, nodeId=instance.id,
                 propMap=gremlin_props, raw=True)
 
-        if isinstance(script_rv, dict) and ERROR_ATTR in script_rv and 'property' in script_rv:
-                raise ValueError( "Duplicate index entries for <%s>.%s" % 
-                                 (instance.__class__.__name__, script_rv['property']))
+        if (isinstance(script_rv, dict) and ERROR_ATTR in script_rv
+            and 'property' in script_rv):
+            raise ValueError("Duplicate index entries for <%s>.%s" % 
+                                (instance.__class__.__name__,
+                                script_rv['property']))
         elif isinstance(script_rv, dict) and 'data' in script_rv:
             #returned a node (TODO #128 error passing generalization)
-            NodeModel._update_values_from_dict(instance, script_rv['data'], clear=True)
+            NodeModel._update_values_from_dict(instance, script_rv['data'],
+                                               clear=True)
         else:
-            raise ValueError('Unexpected response from server: %s' % str(script_rv))
+            raise ValueError('Unexpected response from server: %s' %
+                             str(script_rv))
         
-    NodeModel._save_properties = staticmethod(_save_) #TODO this needs to be revised. I hope there's a better way.
+    #TODO this needs to be revised. I hope there's a better way.
+    NodeModel._save_properties = staticmethod(_save_) 
     del _save_
 
     def __get__(self, instance, cls=None):
@@ -404,7 +434,7 @@ class BoundProperty(AttrRouter):
         else:
             try:
                 values = BoundProperty._values_of(instance)
-                values[self.__propname] = val = self._property.from_neo(underlying[self.__propname])
+                values[self.__propname] = val = self._property.to_python(underlying[self.__propname])
                 return val
             except: # no value set on node
                 pass
@@ -413,6 +443,9 @@ class BoundProperty(AttrRouter):
     def _get_val_from_obj(self, obj):
         return self.__get__(obj)
 
+    def value_from_object(self, obj):
+        return self._get_val_from_obj(obj)
+
     def value_to_string(self, obj):
         #TODO not sure if this method plays a bigger role in django
         return str(self.__get__(obj))
@@ -441,7 +474,6 @@ class BoundProperty(AttrRouter):
         return (old, value)
 
 class StringProperty(Property):
-
     #since strings don't have a natural max, this is an arbitrarily high utf-8
     #string. this is necessary for gt string queries, since Lucene range
     #queries (prior 4.0) don't support open-ended ranges
@@ -453,6 +485,8 @@ class StringProperty(Property):
             kwargs.setdefault('indexed_fulltext', True)
             kwargs.setdefault('indexed_range', True)
         super(StringProperty, self).__init__(**kwargs)
+        self.max_length = max_length
+        self.min_length = min_length
         if max_length is not None:
             self.validators.append(validators.MaxLengthValidator(max_length))
         if min_length is not None:
@@ -461,10 +495,18 @@ class StringProperty(Property):
     def to_neo(cls, value):
         return unicode(value)
 
+    def formfield(self, **kwargs):
+        defaults = dict(kwargs)
+        if self.max_length is not None:
+            defaults['max_length'] = self.max_length
+        return super(StringProperty, self).formfield(**defaults)
+
 class EmailProperty(StringProperty):
     #TODO docstring
     default_validators = [validators.validate_email]
 
+    formfield = formfields.EmailField
+
 class URLProperty(StringProperty):
     #TODO docstring
     def __init__(self, verify_exists=False, **kwargs):
@@ -472,6 +514,11 @@ class URLProperty(StringProperty):
         super(URLProperty, self).__init__(**kwargs)
         self.validators.append(validators.URLValidator(verify_exists=verify_exists))
 
+    def formfield(self, **kwargs):
+        defaults = {'form_class':formfields.URLField}
+        defaults.update(kwargs)
+        return super(URLProperty, self).formfield(**defaults)
+
 class IntegerProperty(Property):
     """
     A 64-bit integer, akin to Django's `BigIntegerField`.
@@ -510,7 +557,14 @@ class IntegerProperty(Property):
         """
         return """{ i -> (i < 0?'-':'0') + String.format('%019d',i)} """
 
+    def formfield(self, **kwargs):
+        defaults = {'form_class':formfields.IntegerField}
+        defaults.update(kwargs)
+        return super(IntegerProperty, self).formfield(**defaults)
+
 class AutoProperty(IntegerProperty):
+    editable = False
+
     def __init__(self, *args, **kwargs):
         kwargs['auto'] = True
         kwargs['auto_default'] = 1
@@ -531,10 +585,11 @@ class AutoProperty(IntegerProperty):
         """
         return """{ i -> i + 1}"""
 
-class DateProperty(Property):
-    __format = '%Y-%m-%d'
+    def formfield(self, **kwargs):
+        return None
 
-    ansi_date_re = re.compile(r'^\d{4}-\d{1,2}-\d{1,2}$')
+@borrows_methods(fields.DateField, ('to_python',))
+class DateProperty(Property):
 
     default_error_messages = {
         'invalid': _('Enter a valid date in YYYY-MM-DD format.'),
@@ -552,58 +607,17 @@ class DateProperty(Property):
             kwargs['blank'] = True
         if kwargs.get('indexed', False):
             kwargs['indexed_range'] = True
-        Property.__init__(self, **kwargs)
-
-    @classmethod
-    def __parse_date_string(cls, value):
-        if not cls.ansi_date_re.search(value):
-            raise exceptions.ValidationError(cls.default_error_messages['invalid'])
-        # Now that we have the date string in YYYY-MM-DD format, check to make
-        # sure it's a valid date.
-        # We could use time.strptime here and catch errors, but datetime.date
-        # produces much friendlier error messages.
-        year, month, day = map(int, value.split('-'))
-        try:
-            value = datetime.date(year, month, day)
-        except ValueError, e:
-            msg = cls.default_error_messages['invalid_date'] % _(str(e))
-            raise exceptions.ValidationError(msg)
-
-        return value
-
-    @classmethod
-    def _format_date(cls, value, format_string=None):
-        #TODO obviously would prefer strftime, but it couldn't do year < 1900-
-        #this should be replaced
-        if not format_string:
-            format_string = cls.__format
-        return format_string.replace('%Y', str(value.year).zfill(4))\
-                     .replace('%m', str(value.month).zfill(2))\
-                     .replace('%d', str(value.day).zfill(2))
-
-    def from_neo(self, value):
-        if value is None or value == '':
-            return None
-        if isinstance(value, datetime.datetime):
-            return value.date()
-        if isinstance(value, datetime.date):
-            return value
-
-        return self.__parse_date_string(value)
+        super(DateProperty, self).__init__(**kwargs)
 
     def to_neo(self, value):
-        result = None
-
         if value is None:
-            return ''
-        if isinstance(value, datetime.datetime):
-            result = value
+            return value
+        elif isinstance(value, datetime.datetime):
+            return value.date().isoformat()
         elif isinstance(value, datetime.date):
-            result = value
+            return value.isoformat()
         else:
-            result = self.__parse_date_string(value)
-
-        return self._format_date(result)
+            return unicode(value)
 
     def pre_save(self, model_instance, add, attname):
         if self.auto_now or (self.auto_now_add and add):
@@ -611,183 +625,43 @@ class DateProperty(Property):
             setattr(model_instance, attname, value)
             return value
         else:
-            return super(DateProperty, self).pre_save(model_instance, add, attname)
+            return super(DateProperty, self).pre_save(model_instance, add,
+                                                      attname)
 
+@borrows_methods(fields.DateTimeField, ('to_python',))
 class DateTimeProperty(DateProperty):
-    __format = '%Y-%m-%d %H:%M:%S.%f'
-
-    default_error_messages = {
-        'invalid': _(u'Enter a valid date/time in YYYY-MM-DD HH:MM[:ss[.uuuuuu]] format.'),
-    }
+    default_error_messages = fields.DateTimeField.default_error_messages
 
     MAX=datetime.datetime.max
     MIN=datetime.datetime.min
 
-    @classmethod
-    def _format_datetime(cls, value, format_string=None):
-        if not format_string:
-            format_string = cls.__format
-        time_string = cls._format_date(value, format_string)
-        return time_string.replace('%H', str(value.hour).zfill(2))\
-                          .replace('%M', str(value.minute).zfill(2))\
-                          .replace('%S', str(value.second).zfill(2))\
-                          .replace('%f', str(value.microsecond).zfill(6))\
-
-    @classmethod
-    def __parse_datetime_string(cls, value):
-        try: # Try converting with microseconds
-            result = datetime.datetime.strptime(value, cls.__format)
-        except ValueError:
-            try: # Try without microseconds.
-                result = datetime.datetime.strptime(value, '%Y-%m-%d %H:%M:%S')
-            except ValueError: # Try without hour/minutes.
-                try: # Try without seconds.
-                    result = datetime.datetime.strptime(value, '%Y-%m-%d %H:%M')
-                except ValueError: # Try without hour/minutes/seconds.
-                    try:
-                        result = datetime.datetime.strptime(value, '%Y-%m-%d')
-                    except ValueError:
-                        raise exceptions.ValidationError(cls.default_error_messages['invalid'])
-
-        return result
-
-    def from_neo(self, value):
-        if value is None or value == '':
-            return None
-
-        if isinstance(value, datetime.datetime):
-            return value
-        if isinstance(value, datetime.date):
-            return datetime.datetime(value.year, value.month, value.day)
-
-        return self.__parse_datetime_string(value)
-
     def to_neo(self, value):
-        result = None
-
         if value is None:
-            return ''
-        if isinstance(value, datetime.datetime):
-            result = value
+            return value
         elif isinstance(value, datetime.date):
-            result = datetime.datetime(value.year, value.month, value.day)
+            if not isinstance(value, datetime.datetime):
+                value = datetime_safe.new_datetime(value)
+            if settings.USE_TZ and timezone.is_naive(value):
+                default_timezone = timezone.get_default_timezone()
+                value = timezone.make_aware(value, default_timezone)
+            return value.isoformat()
         else:
-            result = self.__parse_datetime_string(value)
-
-        return self._format_datetime(result)
+            # TODO raise error
+            pass
 
     def to_neo_index(self, value):
-        return self.to_neo(value).replace(' ','-')
+        cleaned = self.to_neo(value)
+        if cleaned is not None:
+            return cleaned.replace(' ','-')
 
     def pre_save(self, model_instance, add, attname):
         if self.auto_now or (self.auto_now_add and add):
-            value = datetime.datetime.now()
+            value = timezone.now()
             setattr(model_instance, attname, value)
             return value
         else:
-            return super(DateTimeProperty, self).pre_save(model_instance, add, attname)
-
-class DateTimeTZProperty(DateTimeProperty):
-    '''
-    DateTimeProperty that can store and retrieve timezone-aware datetimes.
-    '''
-    __format = '%Y-%m-%d %H:%M:%S.%f %z'
-
-    @classmethod
-    def _format_offset(cls, offset_timedelta):
-        '''
-        Produce a timezone offset string (+/- HHMM) from a timedelta.
-        '''
-        try:
-            seconds = offset_timedelta.total_seconds()
-        except AttributeError:
-            # total_seconds method is only available from 2.7 up
-            td = offset_timedelta
-            days_to_secs = td.days * 24 * 3600.0
-            secs_to_micro = (td.seconds + days_to_secs) * (10 ** 6)
-            seconds = (td.microseconds + secs_to_micro) / (10 ** 6)
-        mins = seconds / 60
-        hrs = mins / 60
-        mins = mins % 60
-        return '%+03d%02d' % (hrs, mins)
-
-    @classmethod
-    def _parse_tz(cls, tz_str=None):
-        '''
-        Read a timezone string of the form '+0000' and return a timezone
-        object. If not given, just return UTC.
-        '''
-        tz_str = tz_str.strip() if tz_str else ''  # Ensure no whitespace
-        if (not tz_str) or (tz_str == '+0000') or (tz_str == '-0000'):
-            # Shortcut for the common case where it's UTC, or default
-            return tzutc()
-        # Otherwise, pull out the hours and minutes and construct a
-        # tzoffset(), which requires an offset in seconds
-        hrs = int(tz_str[1:3])
-        mins = int(tz_str[3:5])
-        mult = -1 if (tz_str[0] == '-') else 1
-        offset = mult * ((hrs * 3600) + (mins * 60))
-        return tzoffset('LOCAL', offset)
-
-    @classmethod
-    def _format_datetime_with_tz(cls, value):
-        '''
-        Format a datetime (e.g. for storage in Neo4j) with a timezone offset
-        appended as +/- HHMM.
-        '''
-        formatted = cls._format_datetime(value, cls.__format)
-        if value.utcoffset() is not None:
-            offset_string = cls._format_offset(value.utcoffset())
-        else:
-            offset_string = ""
-        return formatted.replace("%z", offset_string).strip()
-
-    @classmethod
-    def __parse_datetime_string_with_tz(cls, value):
-        '''
-        Parse a stringified datetime into a datetime object, first trying to
-        read a timezone (if one is provided in our format). Uses the superclass
-        method to parse the actual string, and adds any timezone information
-        at the end.
-        '''
-        try:
-            # Try converting with timezone offset. Since strptime is decidedly
-            # inconsistent with support for '%z', this must be done manually:
-            # if a '+HHMM' is present, it'll form the last five characters of
-            # the string
-            dt_val, tz_str = value[:-5], value[-5:]
-            dt_val = dt_val.strip()  # ensure no trailing whitespace
-            tz_info = cls._parse_tz(tz_str)
-        except ValueError:
-            tz_info = None
-            dt_val = value
-        # HACK: Relies on CPython 2.x's double-underscore name-mangling!
-        dt = cls._DateTimeProperty__parse_datetime_string(dt_val)
-        return dt.replace(tzinfo=tz_info)
-
-    def from_neo(self, value):
-        if value is None or value == '':
-            return None
-        if isinstance(value, datetime.datetime):
-            return value
-        if isinstance(value, datetime.date):
-            return datetime.datetime(value.year, value.month, value.day)
-
-        return self.__parse_datetime_string_with_tz(value)
-
-    def to_neo(self, value):
-        result = None
-
-        if value is None:
-            return ''
-        if isinstance(value, datetime.datetime):
-            result = value
-        elif isinstance(value, datetime.date):
-            result = datetime.datetime(value.year, value.month, value.day)
-        else:
-            result = self.__parse_datetime_string_with_tz(value)
-
-        return self._format_datetime_with_tz(result)
+            return super(DateProperty, self).pre_save(model_instance, add,
+                                                      attname)
 
 class ArrayProperty(Property):
     __metaclass__ = ABCMeta
@@ -823,8 +697,7 @@ class ArrayProperty(Property):
         self.use_string = kwargs.get("use_string", False)
         self.token = kwargs.get("token", ",")
         self.escape_token = kwargs.get("escape_token", "+")
-        self.token_regex = re.compile(
-            "(?<!%s)%s" % (re.escape(self.escape_token), self.token))
+        self.token_regex = "(?<!%s)%s" % (re.escape(self.escape_token), self.token)
 
     def get_default(self):
         if self.use_string:
@@ -834,7 +707,7 @@ class ArrayProperty(Property):
 
     def from_neo(self, value):
         if value and not isinstance(value, (tuple, list)) and self.use_string:
-            array_values = self.token_regex.split(value)
+            array_values = re.split(self.token_regex, value)
             for i, v in enumerate(array_values):
                 array_values[i] = v.replace(
                     "%s%s" % (self.escape_token, self.token), self.token)
@@ -871,3 +744,10 @@ class IntArrayProperty(ArrayProperty):
     default_validators = [validate_int_array]
 
     member_to_neo_index = IntegerProperty.to_neo_index.im_func
+
+class BooleanProperty(Property):
+    def to_neo(self, value):
+        return bool(value)
+
+    def from_neo(self, value):
+        return bool(value)
