@@ -1,6 +1,8 @@
 import os
 BASE_DIR = os.path.dirname(os.path.abspath(__file__))
 
+AUTHENTICATION_BACKENDS = ('neo4django.auth.backends.NodeModelBackend',)
+
 NEO4J_DATABASES = {
     'default' : {
         'HOST':'localhost',
@@ -15,9 +17,23 @@ NEO4J_DATABASES = {
         'PORT':7474,
         'ENDPOINT':'/db/data',
         'CLIENT': 'neo4django.tests.neo4jclient_tests.MyGraphDatabase',
+        'OPTIONS':{
+            'CLEANDB_URI': '/cleandb/supersecretdebugkey!',
+        },
     },
 }
 
+NEO4J_TEST_DATABASES = {
+    'default' : {
+        'HOST':'localhost',
+        'PORT':7474,
+        'ENDPOINT':'/db/data',
+        'OPTIONS':{
+            'CLEANDB_URI': '/cleandb/supersecretdebugkey!',
+        }
+    }
+}
+
 DATABASES = {
     'default': {
         'ENGINE': 'django.db.backends.sqlite3',
@@ -27,6 +43,8 @@ DATABASES = {
 
 DATABASE_ROUTERS = ['neo4django.utils.Neo4djangoIntegrationRouter']
 
+USE_TZ = True
+
 INSTALLED_APPS = (
     'neo4django.tests',   
 )
