@@ -96,8 +96,6 @@ tests=[
 	'test_post_save',
 	'test_pre_delete',
 	'test_post_delete',
-	'test_pre_queryset_delete',
-	'test_post_queryset_delete',
 	'test_in_bulk_not_found',
 	'test_none',
 	'test_relationship_none',
@@ -111,7 +109,21 @@ tests=[
 	'test_relationship_delete',
 	'test_rel_cache',
 	'test_conflicting_rel_types',
-    'test_syncdb']
+	'test_syncdb',
+	'test_model_pickling',
+	'touch_test_db',
+	'rm_test_db',
+	'test_spanning_lookup',
+	'test_order_by',
+	'test_count',
+	'test_aggregate_count',
+	'test_exists',
+    'test_aggregate_avg',
+    'test_aggregate_sum',
+    'test_aggregate_max_min',
+    'test_auth',
+    'test_modelform',
+    'test_query_type']
 should_fail=[
 	'test_dates',
 	'test_filter_iexact',
@@ -120,4 +132,5 @@ should_fail=[
 	'test_url_array_property_validator',
 	'test_type_query',
 	'test_model_casting_validation',
-	'test_array_use_strings']
+	'test_array_use_strings',
+	'test_spanning_lookup']
